package com.streambase.contrib.map;

import java.util.Properties;

import com.bbn.openmap.LayerHandler;
import com.bbn.openmap.MapBean;
import com.bbn.openmap.MapHandler;
import com.bbn.openmap.gui.EmbeddedNavPanel;
import com.bbn.openmap.gui.EmbeddedScaleDisplayPanel;
import com.bbn.openmap.gui.MapPanel;
import com.bbn.openmap.gui.OpenMapFrame;
import com.bbn.openmap.gui.OverlayMapPanel;
import com.bbn.openmap.gui.ToolPanel;
import com.bbn.openmap.layer.learn.BasicLayer;
import com.bbn.openmap.layer.shape.ShapeLayer;
import com.bbn.openmap.proj.coords.LatLonPoint;

public class EarthquakeMap {
	
	// Create a Swing frame. The OpenMapFrame knows how to use
    // the MapHandler to locate and place certain objects.
	private static OpenMapFrame frame = null; 
    // The BasicMapPanel automatically creates many default
    // components, including the MapBean and the MapHandler.
    private static MapPanel mapPanel;
    // The basic component of the local map
    private static MapBean mapBean;
    // Create a new set of earthquake points
    private static volatile PointLayer pointLayer;
    private static MapHandler mapHandler;
    // Create a ShapeLayer to show world political boundaries.
    // Set the properties of the layer. This assumes that the
    // datafiles "dcwpo-browse.shp" and "dcwpo-browse.ssx" are
    // in a path specified in the CLASSPATH variable. 
    private static ShapeLayer shapeLayer;
    // Determines whether we have instantiated the graphics yet
    private static boolean instantiation = false;      
    private static final double DIVISOR = 100;
    private static float centerLatitude;
    private static float centerLongitude;
    private static float earthquakeRadius;
    
    public static boolean initEarthquakeWorld(double latitude, double longitude){    	
    	try{	    	
	    	// Create new variables of the components that we need
	    	pointLayer = new PointLayer();	    	
	        shapeLayer = new ShapeLayer();
	               
	        // Set the properties of the shape layer
	        Properties shapeLayerProps = new Properties();
	        shapeLayerProps.put("prettyName", "Political Solid");
	        shapeLayerProps.put("lineColor", "000000");
	        shapeLayerProps.put("fillColor", "BDDE83");
	        shapeLayerProps.put("shapeFile", "Data/dcwpo-browse.shp");
	        shapeLayerProps.put("spatialIndex", "Data/dcwpo-browse.ssx");            
	        shapeLayer.setProperties(shapeLayerProps);
	        
	        // Add the first earthquake
	        pointLayer.addEarthquake((float) latitude, (float) longitude);
	        
	        // Set the instantiation option to be true
	        instantiation = true;
	        return true;
    	} catch (Exception e){
    		return false;
    	}
    }
    
    public static boolean initEarthquakeLocal(double latitude, double longitude, double centerLat, double centerLon, double radius){    	
    	try{	    	
	    	// Create new variables of the components that we need
	    	pointLayer = new PointLayer();	    	
	        shapeLayer = new ShapeLayer();
	
	        centerLatitude = (float) centerLat;
	        centerLongitude = (float)centerLon;
	        earthquakeRadius = (float) radius;;
	       
	        // Set the properties of the shape layer
	        Properties shapeLayerProps = new Properties();
	        shapeLayerProps.put("prettyName", "Political Solid");
	        shapeLayerProps.put("lineColor", "000000");
	        shapeLayerProps.put("fillColor", "BDDE83");
	        shapeLayerProps.put("shapeFile", "Data/dcwpo-browse.shp");
	        shapeLayerProps.put("spatialIndex", "Data/dcwpo-browse.ssx");            
	        shapeLayer.setProperties(shapeLayerProps);
	        
	        // Add the first earthquake
	        pointLayer.addEarthquake((float) latitude, (float) longitude);
	        
	        // Set the instantiation option to be true
	        instantiation = true;
	        return true;
    	} catch (Exception e){
    		return false;
    	}
    }
    
    /**
     * Checks whether the earthquake map was instantiated    
     * @return true or false
     */
    public static boolean getInstance(){
    	return instantiation;
    }
    
    /**
	 * Adds a new coordinate to the earthquake list 
     * @param lat The points latitude
     * @param lng The points longitude
     */
    public static boolean addEarthquake(double latitude, double longitude){
    	try {
    		pointLayer.addEarthquake((float) latitude, (float) longitude);
    		return true;
    	} catch (Exception e) {
    		return false;
    	}
    }
    
    /**
     * Draw the actual graphics
     */
    public static boolean drawGraphicsWorld(){
    	try {
    		if (frame != null && frame.isShowing())
    				return false;
    		
            // Create a MapBean
    		mapBean = new MapBean();

            // Create a new OpenMap frame and set its parameters
	    	frame = new OpenMapFrame("StreamBase Earthquake Map");	    	
	    	frame.setResizable(false);
	    	frame.setSize(640, 480);

	    	// Add the background to the map handler
    		shapeLayer.setVisible(true);
    		mapBean.add(shapeLayer);
    		
	        // Add the earthquake points to the map handler
    		pointLayer.createGraphics();
    		pointLayer.setVisible(true);
    		mapBean.add(pointLayer);
    		
            // Add the map to the frame
    		frame.getContentPane().add(mapBean);    	
    		
    		// Set the frame to visible
    		frame.setVisible(true);
    		return true;	    	
    	} catch (Exception e){
    		return false;
    	}
    }
    
    
    /**
     * Draw the actual graphics
     */
    public static boolean drawGraphicsLocal(){
    	try {    
    		if (frame != null && frame.isShowing())
				return false;
    		
    		// Create a new map panel
	        mapPanel = new OverlayMapPanel();
	        
	        // Get the default MapHandler the BasicMapPanel created.
            mapHandler = mapPanel.getMapHandler();
	        
            // Get the default MapHandler the BasicMapPanel created.
            mapBean = mapPanel.getMapBean();
                        
            // Set the map's center              
            mapBean.setCenter(new LatLonPoint.Double(centerLatitude,centerLongitude));            
            // Set the map's scale 
            mapBean.setScale((float)(10000000 * Math.max(earthquakeRadius/DIVISOR, 1)));
            // Set the map's size
            mapBean.setSize(640, 480);
	    		        
	        mapHandler.add(new LayerHandler());
            // Add a ToolPanel for widgets on the north side of the map.
            mapHandler.add(new ToolPanel());
            // Add navigation tools over the map
            mapHandler.add(new EmbeddedNavPanel());
            // Add scale display widget over the map
            mapHandler.add(new EmbeddedScaleDisplayPanel());

	        // Create a new frame and set its parameters
	    	frame = new OpenMapFrame("StreamBase Earthquake Map");
	    	frame.setResizable(false);
	    	frame.setSize(640, 480);
	    	
	    	// Add the frame to the map handler
	    	mapHandler.add(frame);
	    	frame.setVisible(true);
	    	
	    	// Add the background to the map handler
	        shapeLayer.setVisible(true);	      
	        mapHandler.add(shapeLayer);
	        
	        // Add the earthquake points to the map handler
	    	pointLayer.createGraphics();
	    	pointLayer.setVisible(true);
	    	mapHandler.add(pointLayer);		    	
            mapHandler.add(new BasicLayer());            
	    	return true;	    	
    	} catch (Exception e){
    		return false;
    	}
    }
    
    /** 
     * Clear all the earthquakes saved in the GUI list
     */
    public static boolean resetEarthquakes(){
    	try{    		   		
    		pointLayer.clearEarthquakes();    		
    		return true;
    	} catch (Exception e){
    		return false;   		
    	}
    }  
}